//
// Copyright 1994 by AT&T Global Information Solutions Co.
// Dayton, Ohio U.S.A. - All rights reserved.
// AT&T Global Information Solutions Company
//
// Description:
//
// Modifications:
//  02/14/94 - WRC - Created
//

#include "stdafx.h"
#include "cxole.h"

#include <olestd.h>
#include <oleimpl.h>
#include <limits.h>

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC(CxOleDocFile, CObject)

//---------------------------PUBLIC
// CxOleDocFile::CxOleDocFile
//
CxOleDocFile::CxOleDocFile ()
{
	TRACE0("CX.LIB - CxOleDocFile::CxOleDocFile () - Start\n");
	
	m_pRootStg = NULL;
}


//----------------------------PUBLIC
// CxOleDocFile::~CxOleDocFile
//
CxOleDocFile::~CxOleDocFile()
{
	TRACE0("CX.LIB - CxOleDocFile::~CxOleDocFile () - Start\n");

	if (m_pRootStg != NULL)
	{
		Close();
		ASSERT(m_pRootStg == NULL);
	}

}


//--------------------------PUBLIC
// CxOleDocFile::AssertValid
//
#ifdef _DEBUG
void CxOleDocFile::AssertValid () const
{
	TRACE0("CX.LIB - CxOleDocFile::AssertValid () - Start\n");
}
#endif


//--------------------PUBLIC
// CxOleDocFile::Close
//
void CxOleDocFile::Close ()
{
	TRACE0("CX.LIB - CxOleDocFile::Close () - Start\n");
	
	ASSERT_VALID(this);
	ASSERT(m_pRootStg != NULL);
	
	if (m_pRootStg != NULL)
	{
		m_pRootStg->Close ();
		delete m_pRootStg;
		m_pRootStg = NULL;
	}
}


//----------------------------PUBLIC
// CxOleDocFile::CreateDocFile
//
BOOL CxOleDocFile::CreateDocFile (const char * pszFilename, DWORD dwOpenMode,
	CFileException * pError)
{
	TRACE0("CX.LIB - CxOleDocFile::CreateDocFile () - Start\n");
	
	ASSERT_VALID(this);
	ASSERT(m_pRootStg == NULL);
	ASSERT(AfxIsValidString(pszFilename));
	ASSERT(pError == NULL || AfxIsValidAddress(pError, sizeof(CFileException)));
	
	LPSTORAGE lpRootStg;
	HRESULT hr = ::StgCreateDocfile (pszFilename, dwOpenMode, 0, &lpRootStg);
	if (FAILED(hr) && pError != NULL)
		_AfxFillOleFileException(pError, hr);
		
	ASSERT(FAILED(hr) || lpRootStg != NULL);

	if (!FAILED(hr) && lpRootStg != NULL)
		m_pRootStg = new CxOleStorage (lpRootStg);
	
	return !FAILED(hr);
}


//-------------------PUBLIC
// CxOleDocFile::Dump
//
#ifdef _DEBUG
void CxOleDocFile::Dump (CDumpContext & aDC) const
{
}
#endif


//--------------------PUBLIC
// CxOleDocFile::Flush
//
void CxOleDocFile::Flush ()
{
	TRACE0("CX.LIB - CxOleDocFile::Flush () - Start\n");
	
	ASSERT_VALID(this);
	ASSERT(m_pRootStg != NULL);

	if (m_pRootStg != NULL)
		m_pRootStg->Flush ();
}


//----------------------------PUBLIC
// CxOleDocFile::GetProfileInt
//
UINT CxOleDocFile::GetProfileInt (const char* pszSection, const char* pszEntry,
	int iDefault, CFileException* pError)
{
	TRACE0("CX.LIB - CxOleDocFile::GetProfileInt () - Start\n");
	
	UINT iRet = iDefault;
	CString aInt;
	char* pszInt = aInt.GetBuffer(10);
	
	int cb = GetProfileString (pszSection, pszEntry, "", pszInt, 10, pError);
	aInt.ReleaseBuffer();
	if (cb != 0)
		iRet = atoi(aInt);
		
	return iRet;
}


//-------------------------------PUBLIC
// CxOleDocFile::GetProfileString
//
int CxOleDocFile::GetProfileString (const char* pszSection, const char* pszEntry,
	const char* pszDefault, char* pszRetBuf, int cbRetBuf, CFileException* pError)
{
	TRACE0("CX.LIB - CxOleDocFile::GetProfileString () - Start\n");

	ASSERT_VALID(this);
	ASSERT(m_pRootStg != NULL);
	ASSERT(AfxIsValidString(pszSection));
	
	CFileException aError;
	if (pError == NULL)
		pError = &aError;

	// Open the storage for this section
	CxOleStorage * pSectionStg = NULL;
	COleStreamFile * pEntryStream = NULL;
	int cbUsed = 0;
	
	// Go ahead and put default in return buffer.
	strcpy(pszRetBuf, pszDefault);
	
	TRY
	{
		pSectionStg = new CxOleStorage;
			
		// Open storage for section.
		if (!pSectionStg->OpenStorage (m_pRootStg->m_lpStorage, pszSection,
			CX_OPEN_READ_STORAGE_DEFAULT, pError))
				AfxThrowFileException (pError->m_cause, pError->m_lOsError);

		if (pszEntry == NULL)
		{
			// Read all entries
			CStringArray rgEntries;
			CString aEntry;

			pSectionStg->EnumElements(STGTY_STREAM, rgEntries);
			for (int iElem = 0; iElem < rgEntries.GetSize(); iElem++)
			{
				aEntry = rgEntries.GetAt(iElem);
				if (cbUsed + aEntry.GetLength() < cbRetBuf)
				{
					memcpy(pszRetBuf+cbUsed, aEntry, aEntry.GetLength()+1);
					cbUsed += aEntry.GetLength()+1;
				}
			} // end for...
			*(pszRetBuf+cbUsed) = '\0';
		}
		else
		{
			// Open the stream with the entry name.
			pEntryStream = new COleStreamFile;
			if (!pEntryStream->OpenStream(pSectionStg->m_lpStorage, pszEntry,
				CFile::modeRead | CFile::shareExclusive, pError))
					AfxThrowFileException(pError->m_cause, pError->m_lOsError);
						
			// Read string to stream.
			UINT cbRead = pEntryStream->Read(pszRetBuf, cbRetBuf);
			pEntryStream->Close ();
			*(pszRetBuf+cbRead) = '\0';
			cbUsed = cbRead;
				
			delete pEntryStream;
			pEntryStream = NULL;
		}

		// Close section storage
		pSectionStg->Close ();
		delete pSectionStg;
		pSectionStg = NULL;

	} // end TRY...
	CATCH_ALL(e)
	{
		if (pEntryStream != NULL)
			delete pEntryStream;
			
		if (pSectionStg != NULL)
			delete pSectionStg;

	}
	END_CATCH_ALL

	return cbUsed;
}


//--------------------------PUBLIC
// CxOleDocFile::GetSections
//
BOOL CxOleDocFile::GetSections (CStringArray& rgSectionsRet)
{
	TRACE0("CX.LIB - CxOleDocFile::GetSections () - Start\n");
	
	ASSERT_VALID(this);
	ASSERT(m_pRootStg != NULL);

	BOOL bSuccess = FALSE;
	
	if (m_pRootStg != NULL)
		bSuccess = m_pRootStg->EnumElements (STGTY_STORAGE, rgSectionsRet);
		
	return bSuccess;
}


//--------------------------PUBLIC
// CxOleDocFile::LoadFromIni
//
BOOL CxOleDocFile::LoadFromIni (const char * pszIniFilename)
{
	TRACE0("CX.LIB - CxOleDocFile::LoadFromIni () - Start\n");

	ASSERT_VALID(this);
	ASSERT(m_pRootStg != NULL);
	ASSERT(AfxIsValidString(pszIniFilename));

	int cbMaxLine = INT_MAX - 2;
	
	CString * pSection = NULL;
	CString * pLine = NULL;
	CString * pEntry = NULL;
	CString * pValue = NULL;

	BOOL bSuccess = TRUE;
	
	// There are no APIs for enumerating the sections in an INI file,
	// so just read the entire file and as sections and entries are
	// found, add them to the compound file.
	
	CStdioFile aIniFile (pszIniFilename, CFile::modeRead | CFile::typeText);

	TRY
	{
		pSection = new CString;
		pLine = new CString;
		pEntry = new CString;
		pValue = new CString;

		char * pszLine = pLine->GetBuffer(cbMaxLine); // Get a buffer to read lines into.
		
		pszLine = aIniFile.ReadString (pszLine, cbMaxLine);
		
		while (pszLine != NULL)
		{
			pLine->ReleaseBuffer ();
			if (pLine->GetLength() != 0)
				*pLine = pLine->Left(pLine->GetLength() - 1); // remove newline
				
			if (!pLine->IsEmpty() && pLine->GetAt(0) == '[')
			{
				*pLine = pLine->Mid(1);
				int ichEnd = pLine->Find(']');
				if (ichEnd != -1)
					*pSection = pLine->Left(ichEnd);
				else
					*pSection = *pLine; // New section
			}
			else if (!pLine->IsEmpty())
			{
				int ichEqual = pLine->Find ('=');
				if (ichEqual != -1)
				{
					*pEntry = pLine->Mid(0, ichEqual);
					*pValue = pLine->Mid(ichEqual+1);
					WriteProfileString (*pSection, *pEntry, *pValue);
				}
			}

			pszLine = pLine->GetBuffer(cbMaxLine);
			pszLine = aIniFile.ReadString (pszLine, cbMaxLine);
		} // end while...
	}
	CATCH_ALL(e)
	{
	}
	END_CATCH_ALL

	delete pLine;
	delete pSection;
	delete pEntry;
	delete pValue;

	return bSuccess;

}


//--------------------------PUBLIC
// CxOleDocFile::OpenDocFile
//
BOOL CxOleDocFile::OpenDocFile (const char * pszFilename, DWORD dwOpenMode,
	CFileException * pError)
{
	TRACE0("CX.LIB - CxOleDocFile::OpenDocFile () - Start\n");
	
	ASSERT_VALID(this);
	ASSERT(m_pRootStg == NULL);
	ASSERT(AfxIsValidString(pszFilename));
	ASSERT(pError == NULL || AfxIsValidAddress(pError, sizeof(CFileException)));
	
	LPSTORAGE lpRootStg;
	HRESULT hr = ::StgOpenStorage (pszFilename, NULL, dwOpenMode, NULL, 0, &lpRootStg);
	if (FAILED(hr) && pError != NULL)
		_AfxFillOleFileException(pError, hr);
		
	ASSERT(FAILED(hr) || lpRootStg != NULL);
	
	if (!FAILED(hr) && lpRootStg != NULL)
		m_pRootStg = new CxOleStorage (lpRootStg);
	
	return !FAILED(hr);
}


//---------------------------------PUBLIC
// CxOleDocFile::WriteProfileString
//
BOOL CxOleDocFile::WriteProfileString (const char * pszSection, const char * pszEntry,
	const char * pszValue, CFileException * pError)
{
	TRACE0("CX.LIB - CxOleDocFile::WriteProfileString () - Start\n");

	ASSERT_VALID(this);
	ASSERT(m_pRootStg != NULL);
	ASSERT(AfxIsValidString(pszSection));
	ASSERT(pError == NULL || AfxIsValidAddress(pError, sizeof(CFileException)));
	
	CFileException aError;
	if (pError == NULL)
		pError = &aError;

	CxOleStorage * pSectionStg = NULL;
	COleStreamFile * pEntryStream = NULL;

	BOOL bSuccess = TRUE;

	TRY
	{
		// If entry is null, then delete the section.
		if (pszEntry == NULL)
		{
			bSuccess = m_pRootStg->DestroyElement (pszSection, pError);
		}
		else
		{
			pSectionStg = new CxOleStorage;
			
			// Open/create storage for section.
			if (!pSectionStg->OpenStorage (m_pRootStg->m_lpStorage, pszSection,
				CX_OPEN_STORAGE_DEFAULT, pError))
				if (pError->m_cause != SCODE_CODE(STG_E_FILENOTFOUND))
					AfxThrowFileException (pError->m_cause, pError->m_lOsError);
				else
					if (!pSectionStg->CreateStorage(m_pRootStg->m_lpStorage,
						pszSection, CX_CREATE_STORAGE_DEFAULT, pError))
							AfxThrowFileException (pError->m_cause, pError->m_lOsError);
					
			if (pszValue == NULL)
			{
				// Delete this "entry"
				bSuccess = pSectionStg->DestroyElement (pszEntry, pError);
			}
			else
			{
				// Create the stream with the entry name.
				pEntryStream = new COleStreamFile;
				if (!pEntryStream->CreateStream(pSectionStg->m_lpStorage, pszEntry,
					CFile::modeWrite | CFile::shareExclusive | CFile::modeCreate, pError))
						AfxThrowFileException(pError->m_cause, pError->m_lOsError);
						
				// Write string to stream.
				pEntryStream->Write (pszValue, strlen(pszValue));
				pEntryStream->Close ();
				
				delete pEntryStream;
				pEntryStream = NULL;
	
			}
			
			// Close section storage
			pSectionStg->Close ();
			delete pSectionStg;
			pSectionStg = NULL;
		} // end if not deleting section.
		
		//m_pRootStg->Flush ();
		
	} // end try
	CATCH_ALL(e)
	{
		bSuccess = FALSE;
		
		if (pEntryStream != NULL)
			delete pEntryStream;
			
		if (pSectionStg != NULL)
			delete pSectionStg;
	}
	END_CATCH_ALL

	return bSuccess;
}


//
// C Interfaces
//
extern "C" {

// Used to keep file open unless flushed or accesses in different mode.
static CxOleDocFile* g_pDocFile = NULL;
static CString g_strDocFilename;
static DWORD g_dwAccessMode = 0xFFFFFFFF;

// This function resolves the profile filename to either the windows
// directory of the filename specified if it includes a path.
void ResolveIniFilename (LPCSTR lpszFilename, CString* pStrFilenameRet)
{
	char szDrive[_MAX_DRIVE];
	char szDir[_MAX_DIR];
	
	_splitpath(lpszFilename, szDrive, szDir, NULL, NULL);

	if (strlen(szDrive) == 0 && strlen(szDir) == 0)
	{
		CString aWindowsDir;
		char * pszWindowsDir = aWindowsDir.GetBuffer (_MAX_PATH);
		UINT cbRet = ::GetWindowsDirectory (pszWindowsDir, _MAX_PATH);
		aWindowsDir.ReleaseBuffer ();
	
		if (aWindowsDir.Right(1) != "\\")
			aWindowsDir = aWindowsDir + "\\";
			
		*pStrFilenameRet = aWindowsDir + lpszFilename;
	}
	else
		*pStrFilenameRet = lpszFilename;

}


UINT CxGetPrivateProfileInt(LPCSTR lpszSection, LPCSTR lpszEntry,
	int iDefault, LPCSTR lpszFilename)
{
	BOOL bSuccess = TRUE;
	CFileException aError;
	
	CString aStrFilename;
	ResolveIniFilename (lpszFilename, &aStrFilename);
	
	// Keep file open and reuse object.
	BOOL bUseCache = FALSE;
	if (g_pDocFile != NULL)
		if (g_dwAccessMode == CX_OPEN_READ_DOCFILE_DEFAULT)
			if (g_strDocFilename == aStrFilename)
				bUseCache = TRUE;

	if (!bUseCache)
	{
		if (g_pDocFile != NULL)
		{
			g_pDocFile->Close ();
			delete g_pDocFile;
			g_pDocFile = NULL;
		}

		g_pDocFile = new CxOleDocFile;
		bSuccess = g_pDocFile->OpenDocFile (aStrFilename, CX_OPEN_READ_DOCFILE_DEFAULT, &aError);
		if (bSuccess)
		{
			g_dwAccessMode = CX_OPEN_READ_DOCFILE_DEFAULT;
			g_strDocFilename = aStrFilename;
		}
	}
			
	// Get integer from file.
	if (bSuccess)
		bSuccess = g_pDocFile->GetProfileInt (lpszSection, lpszEntry, iDefault, &aError);
	
	return bSuccess;

}


int CxGetPrivateProfileString(LPCSTR lpszSection, LPCSTR lpszEntry,
	LPCSTR lpszDefault, LPSTR lpszReturnBuffer, int cbReturnBuffer, LPCSTR lpszFilename)
{
	BOOL bSuccess = TRUE;
	CFileException aError;

	CString aStrFilename;
	ResolveIniFilename (lpszFilename, &aStrFilename);
	
	// Keep file open and reuse object.
	BOOL bUseCache = FALSE;
	if (g_pDocFile != NULL)
		if (g_dwAccessMode == CX_OPEN_READ_DOCFILE_DEFAULT)
			if (g_strDocFilename == aStrFilename)
				bUseCache = TRUE;

	if (!bUseCache)
	{
		if (g_pDocFile != NULL)
		{
			g_pDocFile->Close ();
			delete g_pDocFile;
			g_pDocFile = NULL;
		}

		g_pDocFile = new CxOleDocFile;
		bSuccess = g_pDocFile->OpenDocFile (aStrFilename, CX_OPEN_READ_DOCFILE_DEFAULT, &aError);
		if (bSuccess)
		{
			g_dwAccessMode = CX_OPEN_READ_DOCFILE_DEFAULT;
			g_strDocFilename = aStrFilename;
		}
	}
			
	// Get string from file.
	if (bSuccess)
		bSuccess = g_pDocFile->GetProfileString (lpszSection, lpszEntry, lpszDefault,
			lpszReturnBuffer, cbReturnBuffer, &aError);
	
	return bSuccess;

}


BOOL CxWritePrivateProfileString (const char * pszSection, const char * pszEntry,
	const char * pszString, const char * pszFilename)
{
	BOOL bSuccess;
	CFileException aError;
	
	CString aStrFilename;
	ResolveIniFilename (pszFilename, &aStrFilename);

	// Check if we should flush the cache.
	if (pszSection == NULL && pszEntry == NULL && pszString == NULL)
	{
		if (g_pDocFile != NULL)
		{
			g_pDocFile->Close ();
			delete g_pDocFile;
			g_pDocFile = NULL;
		}
		return TRUE;
	}
	
	// Keep file open and reuse object.
	BOOL bUseCache = FALSE;
	if (g_pDocFile != NULL)
		if (g_dwAccessMode == CX_OPEN_DOCFILE_DEFAULT)
			if (g_strDocFilename == aStrFilename)
				bUseCache = TRUE;

	if (!bUseCache)
	{
		if (g_pDocFile != NULL)
		{
			g_pDocFile->Close ();
			delete g_pDocFile;
			g_pDocFile = NULL;
		}

		g_pDocFile = new CxOleDocFile;
		bSuccess = g_pDocFile->OpenDocFile (aStrFilename, CX_OPEN_DOCFILE_DEFAULT, &aError);
		if (!bSuccess)
			bSuccess = g_pDocFile->CreateDocFile (aStrFilename, CX_CREATE_DOCFILE_DEFAULT, &aError);
		if (bSuccess)
		{
			g_dwAccessMode = CX_OPEN_DOCFILE_DEFAULT;
			g_strDocFilename = aStrFilename;
		}

	}
		
	// Write string to file.
	if (bSuccess)
		bSuccess = g_pDocFile->WriteProfileString (pszSection, pszEntry, pszString);

	return bSuccess;
}
	
}

// cxdocfile.cpp
