//
// Copyright 1994 by AT&T Global Information Solutions Co.
// Dayton, Ohio U.S.A. - All rights reserved.
// AT&T Global Information Solutions Company
//
// Description:
//  Include file for OLE extension classes to MFC.
//
// Modifications:
//  03/21/94 - WRC - Created
//

#ifndef __CXOLE_H__
#define __CXOLE_H__

#include <afxole.h>

// 'C' functions to use doc files like .INI files
extern "C" {
UINT CxGetPrivateProfileInt (LPCSTR lpszSection, LPCSTR lpszEntry,
	int iDefault, LPCSTR lpszFilename);
int CxGetPrivateProfileString(LPCSTR lpszSection, LPCSTR lpszEntry,
	LPCSTR lpszDefault, LPSTR lpszReturnBuffer, int cbReturnBuffer, LPCSTR lpszFilename);
BOOL CxWritePrivateProfileString (LPCSTR lpszSection, LPCSTR lpszEntry,
	LPCSTR lpszString, LPCSTR lpszFilename);
}

// Default access modes.
#define CX_CREATE_DOCFILE_DEFAULT (STGM_TRANSACTED | STGM_READWRITE | STGM_SHARE_EXCLUSIVE | STGM_CREATE)
#define CX_OPEN_DOCFILE_DEFAULT   (STGM_TRANSACTED | STGM_READWRITE | STGM_SHARE_EXCLUSIVE)
#define CX_OPEN_READ_DOCFILE_DEFAULT (STGM_TRANSACTED | STGM_READ | STGM_SHARE_EXCLUSIVE)
#define CX_CREATE_STORAGE_DEFAULT (STGM_TRANSACTED | STGM_READWRITE | STGM_SHARE_EXCLUSIVE | STGM_CREATE)
#define CX_OPEN_STORAGE_DEFAULT   (STGM_TRANSACTED | STGM_READWRITE | STGM_SHARE_EXCLUSIVE)
#define CX_OPEN_READ_STORAGE_DEFAULT (STGM_TRANSACTED | STGM_READ | STGM_SHARE_EXCLUSIVE)

// Forward classes
class CxOleDocFile;
class CxOleStorage;

//
// C++ Class for doc files.
//

//----------------------------------------------------------------------------
// CxOleDocFile
//
class CxOleDocFile : public CObject
{
	DECLARE_DYNAMIC(CxOleDocFile)

// Constructors and Destructors
public:
	CxOleDocFile ();
	~CxOleDocFile ();

// Operations
	// Create a compound file.
	BOOL CreateDocFile (const char * pszFilename,
		DWORD dwOpenFlags = CX_CREATE_DOCFILE_DEFAULT,
		CFileException * pError = NULL);

	// Open a compound file.
	BOOL OpenDocFile (const char * pszFilename,
		DWORD dwOpenFlags = CX_OPEN_DOCFILE_DEFAULT,
		CFileException * pError = NULL);

	// Get integer data from file.
	UINT GetProfileInt (const char* pszSection, const char* pszEntry,
		int iDefault, CFileException* pError = NULL);
		
	// Get string data from file.
	int GetProfileString (const char* pszSection, const char *pszEntry,
		const char* pszDefault, char* pszRetBuf, int cbRetBuf,
		CFileException* pError = NULL);

	// Get all of the sections.
	BOOL GetSections (CStringArray& rgSectionNamesRet);
	
	// Load the contents of an ini file into a compound file.
	BOOL LoadFromIni (const char * pszIniFilename);
	
	// Write a string to the doc file.
	BOOL WriteProfileString (const char * pszSection, const char * pszEntry,
		const char * pszValue, CFileException * pError = NULL);

// Implementation
public:
	virtual void Close ();
	virtual void Flush ();

#ifdef _DEBUG
	virtual void Dump(CDumpContext&) const;
	virtual void AssertValid() const;
#endif

protected:

	CxOleStorage * m_pRootStg;
	
}; // end class CxOleDocFile


//----------------------------------------------------------------------------
// CxOleStorage
//
class CxOleStorage : public CObject
{
	DECLARE_DYNAMIC(CxOleStorage)

// Constructors and Destructors
public:
	CxOleStorage (LPSTORAGE lpStorage = NULL);
	~CxOleStorage ();

// Operations
	// Create a new storage
	BOOL CreateStorage (LPSTORAGE lpParentStg, const char * pszName,
		DWORD dwOpenFlags = CX_CREATE_STORAGE_DEFAULT,
		CFileException * pError = NULL); 
		
	// Delete an storage or stream from the file.
	virtual BOOL DestroyElement (const char * pszName, CFileException * pError = NULL);
	
	// Enumerate the elements in a storage and return the ones
	// that match the specified type.
	BOOL EnumElements (enum tagSTGTY tyElem, CStringArray& rgNamesRet);
	
	// Open a storage.
	BOOL OpenStorage (LPSTORAGE lpParentStg, const char * pszName,
		DWORD dwOpenFlags = CX_OPEN_STORAGE_DEFAULT,
		CFileException * pError = NULL);

// Implementation
public:
	virtual void Close (); // May raise exception.
	virtual void Flush (); // May raise exception.
	
#ifdef _DEBUG
	virtual void Dump(CDumpContext&) const;
	virtual void AssertValid() const;
#endif
	
protected:
	friend class CxOleDocFile;
	LPSTORAGE m_lpStorage;
	BOOL m_bCloseOnDelete;
	
}; // end class CxOleStorage

#endif // __CXOLE_H__

// end cxole.h
