//
// Copyright 1994 by AT&T Global Information Solutions Co.
// Dayton, Ohio U.S.A. - All rights reserved.
// AT&T Global Information Solutions Company
//
// Description:
//
// Modifications:
//  02/14/94 - WRC - Created
//

#include "stdafx.h"
#include "cxole.h"

#include <olestd.h>
#include <oleimpl.h>

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC(CxOleStorage, CObject)

//---------------------------PUBLIC
// CxOleStorage::CxOleStorage
//
CxOleStorage::CxOleStorage (LPSTORAGE lpStorage)
{
	TRACE0("CX.LIB - CxOleStorage::CxOleStorage () - Start\n");
	
	m_lpStorage = lpStorage;
	m_bCloseOnDelete = TRUE;
}


//----------------------------PUBLIC
// CxOleStorage::~CxOleStorage
//
CxOleStorage::~CxOleStorage()
{
	TRACE0("CX.LIB - CxOleStorage::~CxOleStorage () - Start\n");

	// Close storage if it is still open.
	if (m_lpStorage != NULL && m_bCloseOnDelete)
	{
		Close();
		ASSERT(m_lpStorage == NULL);
	}

}


//--------------------------PUBLIC
// CxOleStorage::AssertValid
//
#ifdef _DEBUG
void CxOleStorage::AssertValid () const
{
	CObject::AssertValid ();
}
#endif


//--------------------PUBLIC
// CxOleStorage::Close
//
void CxOleStorage::Close ()
{
	TRACE0("CX.LIB - CxOleStorage::Close () - Start\n");
	
	ASSERT_VALID(this);
	ASSERT(m_lpStorage != NULL);
	
	if (m_lpStorage != NULL)
	{
		Flush();
		_AfxRelease((LPUNKNOWN*)&m_lpStorage);
	}
}


//----------------------------PUBLIC
// CxOleStorage::CreateStorage
//
BOOL CxOleStorage::CreateStorage (LPSTORAGE lpParentStg, const char * pszName,
	DWORD dwOpenFlags, CFileException * pError)
{
	TRACE0("CX.LIB - CxOleStorage::CreateStorage () - Start\n");
	
	ASSERT_VALID(this);
	ASSERT(m_lpStorage == NULL);
	ASSERT(AfxIsValidString(pszName));
	ASSERT(pError == NULL || AfxIsValidAddress(pError, sizeof(CFileException)));

	HRESULT hr = lpParentStg->CreateStorage (pszName, dwOpenFlags, 0, 0, &m_lpStorage);
	if (FAILED(hr) && pError != NULL)
		_AfxFillOleFileException(pError, hr);
		
	ASSERT(FAILED(hr) || m_lpStorage != NULL);

	return !FAILED(hr);
}


//-----------------------------PUBLIC
// CxOleStorage::DestroyElement
//
BOOL CxOleStorage::DestroyElement (const char * pszName, CFileException * pError)
{
	TRACE0("CX.LIB - CxOleStorage::DestroyElement () - Start\n");
	
	ASSERT_VALID(this);
	ASSERT(m_lpStorage != NULL);

	HRESULT hr = m_lpStorage->DestroyElement (pszName);
	if (FAILED(hr) && pError != NULL)
		_AfxFillOleFileException(pError, hr);
		
	ASSERT(FAILED(hr));

	return !FAILED(hr);
}


//-------------------PUBLIC
// CxOleStorage::Dump
//
#ifdef _DEBUG
void CxOleStorage::Dump (CDumpContext & dc) const
{
	CObject::Dump (dc);
	AFX_DUMP1(dc, "\nm_lpStorage = ", m_lpStorage);
}
#endif


//---------------------------PUBLIC
// CxOleStorage::EnumElements
//
BOOL CxOleStorage::EnumElements (enum tagSTGTY tyElem, CStringArray& rgNamesRet)
{
	TRACE0("CX.LIB - CxOleStorage::EnumElements () - Start\n");
	
	ASSERT_VALID(this);
	ASSERT(m_lpStorage != NULL);

	rgNamesRet.RemoveAll (); // Clean out the array
	
	if (m_lpStorage == NULL)
		return FALSE;
		
	LPENUMSTATSTG lpEnum;
	
	HRESULT hr = m_lpStorage->EnumElements(0, NULL, 0, &lpEnum);
	if (FAILED(hr))
		return FALSE;

	ULONG nGot;
	CString aName;
	STATSTG stgStat;
	
	while (GetScode(lpEnum->Next(1, &stgStat, &nGot)) == S_OK)
	{
		if ((stgStat.type & tyElem) != 0)
		{
			aName = stgStat.pwcsName;
			rgNamesRet.Add (aName);
		}
		::OleStdFreeString(stgStat.pwcsName, NULL);
	}
	
	::OleStdVerifyRelease((LPUNKNOWN)lpEnum, NULL);
	
	return TRUE;

}



//--------------------PUBLIC
// CxOleStorage::Flush
//
void CxOleStorage::Flush ()
{
	TRACE0("CX.LIB - CxOleStorage::Flush () - Start\n");
	
	ASSERT_VALID(this);
	ASSERT(m_lpStorage != NULL);

	BOOL bCommit = TRUE;
	STATSTG stgStat;
	HRESULT hr = m_lpStorage->Stat(&stgStat, STATFLAG_NONAME);
	if (hr == NOERROR && ((stgStat.grfMode & (STGM_WRITE | STGM_READWRITE)) == 0))
		bCommit = FALSE;

	if (bCommit)
	{
		hr = m_lpStorage->Commit (STGC_OVERWRITE);
		if (hr != NOERROR)
			_AfxThrowOleFileException(hr);
	}
}


//--------------------------PUBLIC
// CxOleStorage::OpenStorage
//
BOOL CxOleStorage::OpenStorage (LPSTORAGE lpParentStg, const char * pszName,
	DWORD dwOpenFlags, CFileException * pError)
{
	TRACE0("CX.LIB - CxOleStorage::OpenStorage () - Start\n");
	
	ASSERT_VALID(this);
	ASSERT(m_lpStorage == NULL);
	ASSERT(AfxIsValidString(pszName));
	ASSERT(pError == NULL || AfxIsValidAddress(pError, sizeof(CFileException)));

	HRESULT hr = lpParentStg->OpenStorage (pszName, NULL, dwOpenFlags, NULL, 0, &m_lpStorage);
	if (FAILED(hr) && pError != NULL)
		_AfxFillOleFileException(pError, hr);
		
	ASSERT(FAILED(hr) || m_lpStorage != NULL);

	return !FAILED(hr);
}

// cxolestg.cpp
