_ALGORITHM ALLEY COLUMN_
edited by Bruce Schneier
"Genetic Annealing"
by Kenneth Price


Listing One

	  /* GENETIC ANNEALING APPLIED TO BIPARTITIONING A GRAPH */
		  /* An original algorithm by Ken Price */
		  /* written with Microsoft Quick C 2.51 */
			     /* May 5, 1994 */
	/* ------- Bit strings can be displayed if M<=38 ------- */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
int STRINGS[40][200];            /* This array contain the bit strings */
int ENERGY[40];                  /* ENERGY[n] holds bit string energies */
int TMP[200];                    /* Temporary storage during mutation */
float THRESHOLD[40];             /* Holds thresholds.  Must be float */
int L,M,N;                       /* Necklace & population size parameters */
int display_flag=0;
unsigned long view_cycle,gmax,gen;
float C,EXS,EXR,PS,PR,PX=0;      /* These are the control variables */
float average,variance,dE,DE=0;
float rnd1(void);                /* These are the function prototypes */
void input_data(void);
void initialize(void);
void stats(void);
int energy(int);
void swap(int);
void reflect(int);
void splice(int);
void save_copy(int);
void restore_(int);
void display(void);
main()             /* ------- Begin mutation loop ------- */
{
   int n,trial_energy;
   float x;
   input_data();                 /* Input control variables */
   initialize();                 /* Load array with randomized bit strings */
   for (gen=0;gen<gmax;gen++)    /* Run gmax generations */
   {
      for(n=0;n<N;n++)           /* Start of mutation loop */
      {
	 save_copy(n);           /* Save a copy of the current bit string */
	 x=rnd1();               /* Choose a mutation */
	 if (x<=PX)
	    splice(n);           /* A splice is chosen... */
	 else
	    if (x<=PR+PX)
	       reflect(n);       /* or a reflection is chosen... */
	    else
	       swap(n);          /* or else do a random swap */
	 trial_energy=energy(n); /* computes energy of trial configuration */
	 if (trial_energy>THRESHOLD[n])    /* Test trial mutation */
	    restore_(n);         /* Restores old string if mutation fails */
	 else                    /* If mutant passes threshold test... */
	 {
	    DE=DE+THRESHOLD[n]-trial_energy;/* add energy change to bank */
	    THRESHOLD[n]=trial_energy;      /* ...reset the threshold */
	    ENERGY[n]=trial_energy;         /* ...and save a copy of the */
	 }                                  /* mutant's energy (optional) */
      }                          /* Do next string or end mutation loop */

		  /* ------- Begin reheating loop ------- */

      if (DE!=0)                 /* If DE is zero skip reheating loop */
      {
	 dE=DE*C/N;              /* Compute threshold reheating increment */
	 for(n=0;n<N;n++)
	  {
	     THRESHOLD[n]=THRESHOLD[n]+dE;  /* add dE to each threshold */
	  }
	 DE=0;                   /* empty out the energy bank */
      }

		     /* -------End reheating loop ------- */

      if (kbhit()!=0)
	 break;                  /* hit any key to exit program */
      if (gen % view_cycle==0)
      {
	 if (display_flag==1)    /* Print bit strings as text characters...*/
	 {                       /* provided M<=38 (L<=76) and flag is set */
	    display();
	 }
	 stats();                /* Display statistics once per view_cycle */
      }
   }
	   /* ------- Run for another generation if g<gmax ------- */

   stats();                      /* Done. Print average and variance */
   printf("\n   N=%d            M=%d",N,M);
   printf("\n   C=%f    EXS=%f    EXR=%f",C,EXS,EXR);
   printf("\n  PX=%f     PS=%f     PR=%f\n",PX,PS,PR);
}
		      /* ------- End of main() ------- */

		  /* ------- Function definitions ------- */
float rnd1(void)
{
   float x;
   x=rand();                     /* This function returns a random...*/
   x=x/32768;                    /* number, x, such that (0<=x<1) */
   return x;
}
void input_data(void)
{
   unsigned seed;
   printf("\n   Input:");
   printf("\n\n   Maximum number of generations (1<=gmax<2^32) = ");
   scanf("%ld",&gmax);
   printf("                     Population size (1<=N<=40) = ");
   scanf("%d",&N);
   printf("        Size of necklace (2<=M<100, and M even) = ");
   scanf("%d",&M);
   L=M+M;
   printf("                     Cooling constant (0<=C<=1) = ");
   scanf("%f",&C);
   if (N>1)
   {
      printf("                Splicing probability (0<=PX<=1) = ");
      scanf("%f",&PX);
   }
   PR=1-PX;
   if (PX<1)
   {
      printf("       Reflection probability (0<=PR<=%f) = ",PR);
      scanf("%f",&PR);
      if (PR!=0)
      {
	 printf("           Reflection decay constant (0<=EXR<1) = ");
	 scanf("%f",&EXR);
      }
   }
   PS=1-PX-PR;
   printf("                     Probability of random swap = %f\n",PS);
   if (PS>0)
   {
      printf("                  Swap decay constant (0<=EX<1) = ");
      scanf("%f",&EXS);
   }
   printf("                    Random seed (1<=seed<65536) = ");
   scanf("%u",&seed);
   srand(seed);                  /* initialize random number generator */
   printf("         View stats every how many generations? : ");
   scanf("%ld",&view_cycle);
   if (M<=38)
   {
      printf("             Display bit strings? (1=yes, 0=no) :  ");
      scanf("%d",&display_flag);
   }
   printf("\n                            PRESS ANY KEY TO ABORT\n");
   printf("\nComputing...\n");
}
int energy(int n)                /* Computes the energy of the nth string */
{                                /* by XOR-ing connected beads on necklace */
   int i,sum=0;
   for (i=0;i<L-2;i=i+2)
   {
      sum=sum+(STRINGS[n][i]^STRINGS[n][i+1])+(STRINGS[n][i]^STRINGS[n][i+2]);
   }
				 /*  This term connects ends of string */

   sum=sum+(STRINGS[n][0]^STRINGS[n][L-2])+(STRINGS[n][L-2]^STRINGS[n][L-1]);
   return sum;
}
void initialize(void)            /* Initalize the array of bit strings */
{
   int i,j,x,tmp;
   for(i=0;i<N;i++)
   {
      for(j=0;j<L;j++)
      {
	 STRINGS[i][j]=j & 1;    /* Store 010101... in each string to... */
      }                          /* insure an equal number of 0's & 1's */
   }
   for(i=0;i<N;i++)
   {
      for(j=0;j<L;j++)
      {
	 x=rnd1()*L;             /* Next, swap each bit with a random bit */
	 tmp=STRINGS[i][j];      /* from the same string */
	 STRINGS[i][j]=STRINGS[i][x];
	 STRINGS[i][x]=tmp;
      }
      ENERGY[i]=energy(i);       /* Compute & store the string's energy... */
      THRESHOLD[i]=ENERGY[i];    /* then use it to initialize threshold */
   }
}
void stats(void)                 /* Compute population statistics */
{
   int i;
   float de;
   average=0;
   variance=0;
   printf("\nEnergies: ");
   for(i=0;i<N;i++)              /* Computes the average energy... */
   {                             /* and displays the energy of each... */
      average=average+ENERGY[i]; /* bit string */
      printf("%d ",ENERGY[i]);
   }
   average=average/N;            /* This loop computes the variance of... */
   for(i=0;i<N;i++)              /* the population */
   {
      de=ENERGY[i]-average;
      variance=variance+de*de;
   }
   variance=variance/N;          /* Print results to screen */
   printf("\nGeneration = %ld",gen);
   printf("    Average = %f",average);
   printf("    Variance = %f\n",variance);
}
void save_copy(int n)            /* Save a copy of nth string in TMP */
{
   int i;
   for (i=0;i<L;i++)
   {
      TMP[i]=STRINGS[n][i];
   }
}
void restore_(int n)             /* Copy string in TMP to nth string */
{
   int i;
   for (i=0;i<L;i++)
   {
      STRINGS[n][i]=TMP[i];
   }
}
void swap(int n)                 /* Swap a pair of different random bits */
{
   int x1,x2;
   do
   {
      x1=rnd1()*L;               /* Pick first random bit */
      do                         /* Look for a second bit... */
      {                          /* with a complementary value */
      x2=rnd1()*L;
      }  while (STRINGS[n][x1]==STRINGS[n][x2]);
      STRINGS[n][x1]=1-STRINGS[n][x1];         /* Invert bit values */
      STRINGS[n][x2]=1-STRINGS[n][x2];
				 /* Try another swap if random number is */
   }   while (rnd1()<EXS);       /* less than exponential decay constant */
}
void splice(int n)               /* Splice string fragment into nth string */
{
   int x,rs,rb,t=0,sum=0;        /* rb="random bit" & rs="random string" */
   do
   {
      rs=rnd1()*N;               /* Pick a different string to be donor */
   }  while(rs==n);
   rb=rnd1()*L;                  /* Pick a random bit to start splice */
   while (t<L)                              /* Swap no more than L bits */
   {
      x=(rb+t)%L;
      if (STRINGS[n][x]!=STRINGS[rs][x])    /* If target bit not equal to...*/
      {                                     /* doner bit, sum difference...*/
	 sum=sum+STRINGS[n][x]-STRINGS[rs][x];
	 STRINGS[n][x]=1-STRINGS[n][x];     /* and invert the target bit */
	 if (sum==0) return;                /* Stop when number of 1's ...*/
      }                                     /* and 0's are equal (sum=0) */
      t++;
   }                                        /* count bits donated */
}
void reflect(int n)              /* Reflect a sub-string of the nth string */
{
   int x1,x2,t=0,tmp;
   float x;
   x1=rnd1()*L;                  /* Pick a random bit */
   if ((x1 & 1)==0)              /* If it is even...*/
   {
      x2=(x1+L-2)%L;             /* make x2 the bit to its left on the ring */
   }
   else                          /* If x1 is odd...*/
   {
      x1=(x1+1)%L;               /* make x1 next bit to right on ring... */
      x2=(x1+L-4)%L;             /* and make x2 next bit to left on ring */
   }
   do                            /* swap bits at x1 and x2 */
   {
      tmp=STRINGS[n][x1];
      STRINGS[n][x1]=STRINGS[n][x2];
      STRINGS[n][x2]=tmp;
      x1=(x1+1)%L;               /* Move x1 one bit to the left */
      x2=(x2+1+L-4*(x2 & 1))%L;  /* Move x2 one bit to the right */
      t=t+2;                     /* Count number of bits swapped */
   }  while (rnd1()<EXR && t<L); /* Swap next pair if random number...*/
}                                /* is less than the reflection...*/
				 /* decay constant and t<L */
void display(void)
{
   int i,j;                      /* This function prints out bit strings. */
   for (i=0;i<N;i++)             /* Because of display limitations... */
   {                             /* only bit strings shorter than M=40... */
      printf("\n");              /* will be printed to the screen */
      for (j=0;j<L;j++)
      {
	 printf("%i",STRINGS[i][j]);
      }
   }
}
