// inidoc.cpp : implementation of the CIniDoc class
//

#include "stdafx.h"
#include "ini.h"

#include "inidoc.h"

#include "cxole.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CIniDoc

IMPLEMENT_DYNCREATE(CIniDoc, CDocument)

BEGIN_MESSAGE_MAP(CIniDoc, CDocument)
	//{{AFX_MSG_MAP(CIniDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIniDoc construction/destruction

CIniDoc::CIniDoc()
{
	InitMembers ();
}

CIniDoc::~CIniDoc()
{
}

void CIniDoc::CreateTestFiles(CArchive& ar)
{

	// Make sure old files are closed.
	CxWritePrivateProfileString (NULL, NULL, NULL, m_aTestCompIniFile);
	
	InitMembers (); // Set times to zero.
	
	// Copy the opened ini file in the archive to a test ini
	// file that can be modified. We will never modify the
	// original ini file. Next create a test compound ini file
	// and fill it with the sections and entries from the ASCII
	// ini file.
	char szASCIIIniFilename[_MAX_PATH];
	char szCompIniFilename[_MAX_PATH];
	
	GetTempFileName(0, "INI", 1, szASCIIIniFilename);
	GetTempFileName(0, "CF", 1, szCompIniFilename);
	
	m_aTestASCIIIniFile = szASCIIIniFilename;
	m_aTestCompIniFile = szCompIniFilename;
	
	// Copy the original ini to our test file.
	UINT cbMax = 65000;
	char * prgb = new char [cbMax];
	
	UINT cb = ar.Read (prgb, cbMax);
		
	CFile aASCIIFile;
	if (!aASCIIFile.Open(m_aTestASCIIIniFile,
		CFile::modeCreate | CFile::modeWrite | CFile::shareExclusive))
	{
		delete [] prgb;
		return;
	}
	
	aASCIIFile.Write (prgb, cb);
	aASCIIFile.Close ();

	delete [] prgb;

	// Create a compound file and tell it to load its
	// contents from the INI file. Time how long it takes
	CTime aTimeStart = CTime::GetCurrentTime ();
	
	CxOleDocFile aDocFile;
	if (!aDocFile.CreateDocFile(m_aTestCompIniFile))
		return;
		
	aDocFile.LoadFromIni(m_aTestASCIIIniFile);
	aDocFile.Close ();

	CTime aTimeEnd = CTime::GetCurrentTime ();
	CTimeSpan aTimeElapsed = aTimeEnd - aTimeStart;
	m_wSecsToConvert = (int)aTimeElapsed.GetTotalSeconds ();
	
	UpdateAllViews (NULL);
	
	BOOL bOk = CxWritePrivateProfileString ("TestSection", "TestEntry",
		"EntryValue", "TEST.CF");

}


void CIniDoc::DoTimeTest(CIniDoc::TestFlags tyTest)
{
	CStringArray rgSections;
	CStringArray rgEntriesIni;
	CStringArray rgValuesIni;
	CStringArray rgEntriesCF;
	CStringArray rgValuesCF;
	CString aValue;
	int iSection;
	int iEntry;

	// Get all of the sections. Sections are same in both files.
	GetSections(rgSections);
	
	CTimeSpan aCFGetTime(0,0,0,0);
	CTimeSpan aIniGetTime(0,0,0,0);
	CTimeSpan aCFWriteTime(0,0,0,0);
	CTimeSpan aIniWriteTime(0,0,0,0);
	CTime aTimeStart;
	CTime aTimeEnd;
	
	for (iSection = 0; iSection < rgSections.GetSize(); iSection++)
	{
		aTimeStart = CTime::GetCurrentTime();
		GetEntries (rgSections.GetAt(iSection), rgEntriesCF, rgValuesCF, TRUE);
		aTimeEnd = CTime::GetCurrentTime();
		aCFGetTime = aCFGetTime + (aTimeEnd - aTimeStart);
		
		aTimeStart= CTime::GetCurrentTime();
		GetEntries (rgSections.GetAt(iSection), rgEntriesIni, rgValuesIni, FALSE);
		aTimeEnd = CTime::GetCurrentTime();
		aIniGetTime = aIniGetTime + (aTimeEnd - aTimeStart);

		if (tyTest == TEST_WRITE_SHORTER || tyTest == TEST_WRITE_LONGER)
		{
			// Make each entry's value one byte longer
			aTimeStart = CTime::GetCurrentTime();
			for (iEntry = 0; iEntry < rgEntriesCF.GetSize(); iEntry++)
			{
				aValue = rgValuesCF.GetAt(iEntry);
				if (tyTest == TEST_WRITE_LONGER)
					aValue= aValue + "X";
				else if (!aValue.IsEmpty())
					aValue = aValue.Left(aValue.GetLength()-1);
					
				CxWritePrivateProfileString(rgSections.GetAt(iSection),
					rgEntriesCF.GetAt(iEntry), aValue, m_aTestCompIniFile);
			}
			aTimeEnd = CTime::GetCurrentTime();
			aCFWriteTime = aCFWriteTime + (aTimeEnd - aTimeStart);
			
			aTimeStart = CTime::GetCurrentTime();
			for (iEntry = 0; iEntry < rgEntriesIni.GetSize(); iEntry++)
			{
				aValue = rgValuesIni.GetAt(iEntry);
				if (tyTest == TEST_WRITE_LONGER)
					aValue= aValue + "X";
				else if (!aValue.IsEmpty())
					aValue = aValue.Left(aValue.GetLength()-1);
					
				WritePrivateProfileString(rgSections.GetAt(iSection),
					rgEntriesIni.GetAt(iEntry), aValue, m_aTestASCIIIniFile);
			}
			aTimeEnd = CTime::GetCurrentTime();
			aIniWriteTime = aIniWriteTime + (aTimeEnd - aTimeStart);
		}
	} // end for...
	
	switch (tyTest)
	{
		case TEST_GETALL:
			m_wSecsIniGetAll = aIniGetTime.GetSeconds();
			m_wSecsCFGetAll = aCFGetTime.GetSeconds();
			break;
			
		case TEST_WRITE_SHORTER:
			m_wSecsIniShorter = aIniWriteTime.GetSeconds();
			m_wSecsCFShorter = aCFWriteTime.GetSeconds();
			break;
			
		case TEST_WRITE_LONGER:
			m_wSecsIniLonger = aIniWriteTime.GetSeconds();
			m_wSecsCFLonger = aCFWriteTime.GetSeconds();
			break;

	} // end switch
	
	UpdateAllViews (NULL);
}


void CIniDoc::GetEntries(const char* pszSection, CStringArray& rgEntriesRet,
	CStringArray& rgValuesRet, BOOL bGetFromCF)
{
	// Use replacement ini file functions to get all of the entries
	// and their values for the specified section.
	CString aEntry;
	CString aValue;
	
	int cbBuf = 1024*8;
	char* pszRetBuf = new char [cbBuf];
	char* pszEntry = pszRetBuf;
	int cb;
	
	// Read all of the entry names for a section.
	if (bGetFromCF)
		cb = CxGetPrivateProfileString(pszSection, NULL, "", pszRetBuf, cbBuf, m_aTestCompIniFile);
	else
		cb = GetPrivateProfileString(pszSection, NULL, "", pszRetBuf, cbBuf, m_aTestASCIIIniFile);
	
	rgEntriesRet.RemoveAll ();
	rgValuesRet.RemoveAll ();
	
	while (strlen(pszEntry) != 0)
	{
		char* pszValue = aValue.GetBuffer (cbBuf);
		if (bGetFromCF)
			cb = CxGetPrivateProfileString (pszSection, pszEntry, "", pszValue, cbBuf, m_aTestCompIniFile);
		else
			cb = GetPrivateProfileString (pszSection, pszEntry, "", pszValue, cbBuf, m_aTestASCIIIniFile);
				
		aValue.ReleaseBuffer();
		
		if (cb >= 0)
		{
			rgEntriesRet.Add(pszEntry);
			rgValuesRet.Add(aValue);
		}
		
		pszEntry += strlen(pszEntry) + 1;
	} // end while...
}
	
	
void CIniDoc::GetSections(CStringArray& rgNamesRet)
{
	// Make sure doc is closed.
	CxWritePrivateProfileString (NULL, NULL, NULL, m_aTestCompIniFile);
	
	CxOleDocFile aDocFile;
	if (!aDocFile.OpenDocFile(m_aTestCompIniFile))
		return;
		
	aDocFile.GetSections(rgNamesRet);
	aDocFile.Close();
}


void CIniDoc::InitMembers ()
{
	m_wSecsToConvert = 0;
	
	m_wSecsIniLonger = 0;
	m_wSecsIniShorter = 0;
	m_wSecsIniGetAll = 0;
	m_wSecsCFLonger = 0;
	m_wSecsCFShorter = 0;
	m_wSecsCFGetAll = 0;
}


BOOL CIniDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CIniDoc serialization

void CIniDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// Copy ini file to temporary ini file and convert to compound
		// ini file
		CreateTestFiles (ar);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CIniDoc diagnostics

#ifdef _DEBUG
void CIniDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CIniDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CIniDoc commands
